// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file is released under the terms of the CeCILL_V2 license : http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function errmsg = apifun_checkgreq ( funname , var , varname , ivar , thr )
  // Check that the value is greater or equal than a threshold.
  //
  // Calling Sequence
  //   errmsg = apifun_checkgreq ( funname , var , varname , ivar , thr )
  //
  // Parameters
  //   funname : a 1 x 1 matrix of strings, the name of the calling function.
  //   var : a 1 x 1 matrix of valid Scilab data type, the variable
  //   varname : a 1 x 1 matrix of string, the name of the variable
  //   ivar : a 1 x 1 matrix of floating point integers, the index of the input argument in the calling sequence
  //   thr : a matrix of values, the minimum value for the variable #ivar
  //   errmsg : a 1 x 1 matrix of strings, the error message. If there was no error, the error message is the empty matrix.
  //
  // Description
  // This function is designed to be used to design functions where an 
  // input argument is expected to be greater or equal to a threshold.
  // The error is generated if the condition or ( var < thr ) is true.
  // This function can be used for whatever variable type for which 
  // the comparison "<" can be evaluated.
  //
  // Examples
  // // The function takes an argument x such that x>=1.
  // function y = myfunction ( x )
  //   apifun_checkgreq ( "myfunction" , x , "x" , 1 , 1 )
  //   y = sqrt(x-1)
  // endfunction
  // // Calling sequences which work
  // myfunction ( [1.1,2.2,3.3] )
  // // Calling sequences which generate an error
  // myfunction ( [0.2,1] )
  // myfunction ( [1,-1.1,2,4.4] )
  //
  // Authors
  //   Michael Baudin - 2010 - DIGITEO
  //

  [lhsnb,rhsnb]=argn()
  if ( rhsnb <> 5 ) then
    msgfmt = "%s: Unexpected number of input arguments : %d provided while %d are expected."
    errmsg = msprintf(gettext(msgfmt), "apifun_checkgreq", rhsnb,5)
    error(errmsg)
  end
  //
  // Checking type of input arguments
  if ( typeof(funname) <> "string" ) then
    errmsg = msprintf(gettext("%s: Expected a string for input argument %s at input #%d, but got %s instead."), "apifun_checkgreq", "funname" , 1 , funname , typeof(funname) );
    error(errmsg)
  end
  if ( typeof(varname) <> "string" ) then
    errmsg = msprintf(gettext("%s: Expected a constant for input argument %s at input #%d, but got [%s] instead."), "apifun_checkgreq", "varname" , 3 , varname , typeof(varname) );
    error(errmsg)
  end
  if ( typeof(ivar) <> "constant" ) then
    errmsg = msprintf(gettext("%s: Expected a constant for input argument %s at input #%d, but got [%s] instead."), "apifun_checkgreq", "ivar" , 4 , ivar , typeof(ivar) );
    error(errmsg)
  end
  //
  // Checking size of input arguments
  if ( or(size(funname) <> [1 1]) ) then
    strcomp = strcat(string(size(funname))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "apifun_checkgreq", "funname" , 1 , strcomp );
    error(errmsg)
  end
  if ( or(size(varname) <> [1 1]) ) then
    strcomp = strcat(string(size(varname))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "apifun_checkgreq", "varname" , 3 , varname );
    error(errmsg)
  end
  if ( or(size(ivar) <> [1 1]) ) then
    strcomp = strcat(string(size(ivar))," ")
    errmsg = msprintf(gettext("%s: Expected a scalar for input argument %s at input #%d, but got [%s] instead."), "apifun_checkgreq", "ivar" , 4 , strcomp );
    error(errmsg)
  end
  //
  errmsg = []
  if ( or ( var < thr ) ) then
    k = find ( var < thr )
    k = k(1)
    errmsg = msprintf(gettext("%s: Expected that all entries of input argument %s at input #%d are greater or equal than %s, but entry #%d is equal to %s."),funname,varname,ivar,string(thr),k,string(var(k)));
    error(errmsg);
  end
endfunction

